/*
* douluo-ui v1.0.2
* Copyright ©2022-2022 
* Released under the MIT License.
*/
'use strict';

let componentNamespace = 'dl'; // 组件前缀命名只能更新一次，防止重复执行修改组件注册出错

let isUpdated = false; // scss 命名不能通过 js 更改，在 SCSS 编译已经打包好了

const cssNamespace = 'dl';
const statePrefix = 'is-';
function createNamespace(name, {
  prefix
}) {
  if (!isUpdated) {
    isUpdated = true;
    componentNamespace = prefix || componentNamespace; // 更改前缀
  }

  return componentNamespace + name.charAt(0).toUpperCase() + name.substr(1);
}
/**
 * 生成 bem
 * @param {} namespace 命名空间
 * @param {*} block 块
 * @param {*} blockSuffix 块多个单词
 * @param {*} element 元素
 * @param {*} modifier 修饰符
 * @returns
 */

const _bem = (namespace, block, blockSuffix, element, modifier) => {
  let cls = `${namespace}-${block}`;

  if (blockSuffix) {
    cls += `-${blockSuffix}`;
  }

  if (element) {
    cls += `__${element}`;
  }

  if (modifier) {
    cls += `--${modifier}`;
  }

  return cls;
};

const useNamespace = block => {
  const namespace = cssNamespace;

  const b = (blockSuffix = '') => _bem(namespace, block, blockSuffix, '', '');

  const e = element => element ? _bem(namespace, block, '', element, '') : '';

  const m = modifier => modifier ? _bem(namespace, block, '', '', modifier) : '';

  const be = (blockSuffix, element) => blockSuffix && element ? _bem(namespace, block, blockSuffix, element, '') : '';

  const em = (element, modifier) => element && modifier ? _bem(namespace, block, '', element, modifier) : '';

  const bm = (blockSuffix, modifier) => blockSuffix && modifier ? _bem(namespace, block, blockSuffix, '', modifier) : '';

  const bem = (blockSuffix, element, modifier) => blockSuffix && element && modifier ? _bem(namespace, block, blockSuffix, element, modifier) : '';

  const is = (name, ...args) => {
    const state = args.length >= 1 ? args[0] : true;
    return name && state ? `${statePrefix}${name}` : '';
  }; // for css var
  // --el-xxx: value;


  const cssVar = object => {
    const styles = {};

    for (const key in object) {
      styles[`--${namespace}-${key}`] = object[key];
    }

    return styles;
  }; // with block


  const cssVarBlock = object => {
    const styles = {};

    for (const key in object) {
      styles[`--${namespace}-${block}-${key}`] = object[key];
    }

    return styles;
  };

  const cssVarName = name => `--${namespace}-${name}`;

  const cssVarBlockName = name => `--${namespace}-${block}-${name}`;

  return {
    cssNamespace,
    componentNamespace,
    b,
    e,
    m,
    be,
    em,
    bm,
    bem,
    is,
    // css
    cssVar,
    cssVarName,
    cssVarBlock,
    cssVarBlockName
  };
};

// 样式资源包
let chalk = '';

const getThemeCluster = function (theme) {
  const tintColor = (color, tint) => {
    let red = parseInt(color.slice(0, 2), 16);
    let green = parseInt(color.slice(2, 4), 16);
    let blue = parseInt(color.slice(4, 6), 16);

    if (tint === 0) {
      // when primary color is in its rgb space
      return [red, green, blue].join(',');
    } else {
      red += Math.round(tint * (255 - red));
      green += Math.round(tint * (255 - green));
      blue += Math.round(tint * (255 - blue));
      red = red.toString(16);
      green = green.toString(16);
      blue = blue.toString(16);
      return `#${red}${green}${blue}`;
    }
  };

  const shadeColor = (color, shade) => {
    let red = parseInt(color.slice(0, 2), 16);
    let green = parseInt(color.slice(2, 4), 16);
    let blue = parseInt(color.slice(4, 6), 16);
    red = Math.round((1 - shade) * red);
    green = Math.round((1 - shade) * green);
    blue = Math.round((1 - shade) * blue);
    red = red.toString(16);
    green = green.toString(16);
    blue = blue.toString(16);
    return `#${red}${green}${blue}`;
  };

  const clusters = [theme];

  for (let i = 0; i <= 9; i++) {
    clusters.push(tintColor(theme, Number((i / 10).toFixed(2))));
  }

  clusters.push(shadeColor(theme, 0.1));
  return clusters;
};

const getCSSString = function (url) {
  return new Promise(resolve => {
    const xhr = new XMLHttpRequest();

    xhr.onreadystatechange = () => {
      if (xhr.readyState === 4 && xhr.status === 200) {
        chalk = xhr.responseText.replace(/@font-face{[^}]+}/, '');
        resolve();
      }
    };

    xhr.open('GET', url);
    xhr.send();
  });
};

const updateStyle = function (style, oldCluster, newCluster) {
  let newStyle = style;
  oldCluster.forEach((color, index) => {
    newStyle = newStyle.replace(new RegExp(color, 'ig'), newCluster[index]);
  });
  return newStyle;
};

function themeConfigFunc(defaultTheme = '#409EFF') {
  return {
    defaultTheme,
    theme: '',
    updateElementUITheme: async function (options = {}) {
      const {
        version = '2.15.8',
        themeColor,
        appendDom,
        insertAfter,
        cssUrl,
        chalkStyle = 'chalk-style'
      } = options;
      if (typeof themeColor !== 'string') return;
      const oldVal = chalk ? this.theme : this.defaultTheme;
      const themeCluster = getThemeCluster(themeColor.replace('#', ''));
      const originalCluster = getThemeCluster(oldVal.replace('#', ''));

      const getHandler = id => {
        return () => {
          const originalCluster = getThemeCluster(this.defaultTheme.replace('#', ''));
          const newStyle = updateStyle(chalk, originalCluster, themeCluster);
          let styleTag = document.querySelector(id);

          if (!styleTag) {
            styleTag = document.createElement('style');
            styleTag.setAttribute('id', id);

            if (appendDom) {
              if (insertAfter) {
                appendDom.parentNode.insertBefore(styleTag, appendDom.nextSibling);
              } else {
                appendDom.appendChild(styleTag);
              }
            } else {
              document.head.appendChild(styleTag);
            }
          }

          styleTag.innerText = newStyle;
        };
      };

      if (!chalk) {
        const url = cssUrl || `https://unpkg.com/element-ui@${version}/lib/theme-chalk/index.css`;
        await getCSSString(url);
      }

      const chalkHandler = getHandler(chalkStyle);
      chalkHandler();
      this.theme = themeColor;
      const styles = [].slice.call(document.querySelectorAll('style')).filter(style => {
        const text = style.innerText;
        return new RegExp(oldVal, 'i').test(text) && !/Chalk Variables/.test(text);
      });
      styles.forEach(style => {
        const {
          innerText
        } = style;
        if (typeof innerText !== 'string') return;
        style.innerText = updateStyle(innerText, originalCluster, themeCluster);
      });
    }
  };
}

//
var script = {
  name: 'Button',

  data() {
    this.ns = useNamespace('button');
    return {};
  }

};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function () {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "el-button",
    _vm._g(
      _vm._b({ class: [_vm.ns.b()] }, "el-button", _vm.$attrs, false),
      _vm.$listeners
    ),
    [_vm._t("default")],
    2
  )
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

__vue_component__.install = (Vue, options = {}) => {
  Vue.component(createNamespace("button", {
    prefix: options.prefix
  }), __vue_component__);
};

var elementCom = [__vue_component__];

var components = [...elementCom];

const install = (Vue, options = {}) => {
  // 判断是否安装
  if (install.installed) return;
  install.installed = true;
  components.forEach(component => {
    Vue.component(createNamespace(component.name, {
      prefix: options.prefix
    }), component);
  });
}; // 判断调用vue加载器，实现全局加载


if (typeof window !== 'undefined' && window.Vue) {
  install(window.Vue);
} // Vue.use 和按需加载都可以


var index = {
  install,
  themeConfigFunc,
  ...components
};

module.exports = index;
